package com.elitesland.router.service;

import com.elitesland.router.util.GateWayNacosUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.config.GatewayProperties;
import org.springframework.cloud.gateway.config.PropertiesRouteDefinitionLocator;
import org.springframework.cloud.gateway.event.RefreshRoutesEvent;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionRepository;
import org.springframework.cloud.gateway.route.RouteDefinitionWriter;
import org.springframework.cloud.gateway.support.NotFoundException;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.ApplicationEventPublisherAware;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;
import java.util.ArrayList;

/*
 *@Author Horbin.Zhang
 *@Date 2021-02-10 19:29
 *@Description 路由增改删查服务类
 */

@Service
public class DynamicRouteService implements ApplicationEventPublisherAware {

    @Autowired
    private GateWayNacosUtils gateWayNacosUtils;

    @Autowired
    private RouteDefinitionWriter routeDefinitionWriter;

    @Autowired
    GatewayProperties gatewayProperties;

    private ApplicationEventPublisher publisher;

    private ArrayList<String> routeIdInMemory = new ArrayList<>();

    @Override
    public void setApplicationEventPublisher(ApplicationEventPublisher applicationEventPublisher) {
        this.publisher = applicationEventPublisher;
    }

    /**
     * 增加路由
     */
    public String add(RouteDefinition definition) {
        routeDefinitionWriter.save(Mono.just(definition)).subscribe();
        this.publisher.publishEvent(new RefreshRoutesEvent(this));
        return "添加成功！";
    }

    /**
     * 删除路由
     */
    public String delete(String id) {
        Mono<ResponseEntity<Object>> responseEntityMono = this.routeDefinitionWriter.delete(Mono.just(id)).then(Mono.defer(() -> {
            return Mono.just(ResponseEntity.ok().build());
        })).onErrorResume((t) -> {
            return t instanceof NotFoundException;
        }, (t) -> {
            return Mono.just(ResponseEntity.notFound().build());
        });
        responseEntityMono.subscribe();
        this.publisher.publishEvent(new RefreshRoutesEvent(this));
        return "删除成功！";
    }
}